@@ -9,13 +9,8 @@
 #include <libxslt/xsltInternals.h>
 #include <libxslt/extensions.h>
 
-#ifdef HAVE_MATH_H
 #include <math.h>
-#endif
-
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 
 #include "exslt.h"
 
@@ -453,8 +448,6 @@ exsltMathConstantFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlXPathReturnNumber(ctxt, ret);
 }
 
-#if defined(HAVE_STDLIB_H) && defined(RAND_MAX)
-
 /**
  * exsltMathRandom:
  *
@@ -494,10 +487,6 @@ exsltMathRandomFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlXPathReturnNumber(ctxt, ret);
 }
 
-#endif /* defined(HAVE_STDLIB_H) && defined(RAND_MAX) */
-
-#if HAVE_MATH_H
-
 /**
  * exsltMathAbs:
  * @num:  a double
@@ -1029,8 +1018,6 @@ exsltMathExpFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlXPathReturnNumber(ctxt, ret);
 }
 
-#endif /* HAVE_MATH_H */
-
 /**
  * exsltMathRegister:
  *
@@ -1051,16 +1038,12 @@ exsltMathRegister (void) {
     xsltRegisterExtModuleFunction ((const xmlChar *) "lowest",
 				   EXSLT_MATH_NAMESPACE,
 				   exsltMathLowestFunction);
-    /* register other math functions */
     xsltRegisterExtModuleFunction ((const xmlChar *) "constant",
 				   EXSLT_MATH_NAMESPACE,
 				   exsltMathConstantFunction);
-#ifdef HAVE_STDLIB_H
     xsltRegisterExtModuleFunction ((const xmlChar *) "random",
 				   EXSLT_MATH_NAMESPACE,
 				   exsltMathRandomFunction);
-#endif
-#if HAVE_MATH_H
     xsltRegisterExtModuleFunction ((const xmlChar *) "abs",
 				   EXSLT_MATH_NAMESPACE,
 				   exsltMathAbsFunction);
@@ -1097,7 +1080,6 @@ exsltMathRegister (void) {
     xsltRegisterExtModuleFunction ((const xmlChar *) "exp",
 				   EXSLT_MATH_NAMESPACE,
 				   exsltMathExpFunction);
-#endif
 }
 
 /**
@@ -1129,13 +1111,10 @@ exsltMathXpathCtxtRegister (xmlXPathContextPtr ctxt, const xmlChar *prefix)
                                    (const xmlChar *) "lowest",
                                    (const xmlChar *) EXSLT_MATH_NAMESPACE,
                                    exsltMathLowestFunction)
-#ifdef HAVE_STDLIB_H
         && !xmlXPathRegisterFuncNS(ctxt,
                                    (const xmlChar *) "random",
                                    (const xmlChar *) EXSLT_MATH_NAMESPACE,
                                    exsltMathRandomFunction)
-#endif
-#if HAVE_MATH_H
         && !xmlXPathRegisterFuncNS(ctxt,
                                    (const xmlChar *) "abs",
                                    (const xmlChar *) EXSLT_MATH_NAMESPACE,
@@ -1184,7 +1163,6 @@ exsltMathXpathCtxtRegister (xmlXPathContextPtr ctxt, const xmlChar *prefix)
                                    (const xmlChar *) "exp",
                                    (const xmlChar *) EXSLT_MATH_NAMESPACE,
                                    exsltMathExpFunction)
-#endif
         && !xmlXPathRegisterFuncNS(ctxt,
                                    (const xmlChar *) "constant",
                                    (const xmlChar *) EXSLT_MATH_NAMESPACE,
