@@ -10,7 +10,7 @@
 #ifndef __XSLT_LIBXSLT_H__
 #define __XSLT_LIBXSLT_H__
 
-#if defined(_WIN32) && !defined (__CYGWIN__) && !defined (__MINGW32__)
+#if defined(_WIN32) && !defined (__MINGW32__)
 #include <win32config.h>
 #else
 #include "config.h"
